/***********************************************************************/
/*                                                                     */
/*  FILE        : RIIC_RX63N_MASTER.c                                  */
/*  DATE        : Tue, Oct 31, 2006                                    */
/*  DESCRIPTION : Main Program                                         */
/*  CPU TYPE    :                                                      */
/*                                                                     */
/*  NOTE:THIS IS A TYPICAL EXAMPLE.                                    */
/*                                                                     */
/***********************************************************************/
/*******************************************************************************
Includes <System Includes> , "Project Includes"
*******************************************************************************/
#include "typedefine.h"
#include "iodefine.h"
#include <machine.h>
#include <stdio.h>
#include <string.h>

/*******************************************************************************
Macro definitions
*******************************************************************************/
#define MASTER_SENDS_ADR_7_W                 (0x00U) /* Master sends 7 bits address with write */
#define MASTER_SENDS_ADR_7_R                 (0x01U) /* Master sends 7 bits address with read */
#define MASTER_SENDS_DATA                    (0x05U) /* Master sends data */
#define MASTER_SENDS_END                     (0x06U) /* Master sends prepares to stop */
#define MASTER_SENDS_STOP                    (0x07U) /* Master sends stopping */
#define MASTER_RECEIVES_START                (0x08U) /* Master receive to start */
#define MASTER_RECEIVES_DATA                 (0x09U) /* Master is receiving data */
#define MASTER_RECEIVES_STOPPING             (0x0AU) /* Master receive prepares to stop */
#define MASTER_RECEIVES_STOP                 (0x0BU) /* Master receive stopping */
#define MASTER_RECEIVE                       (0x0CU) /* Master receive mode */
#define MASTER_TRANSMIT                      (0x0DU) /* Master transmit mode */
#define MASTER_RECEIVES_RESTART              (0x0EU) /* Master receive restarting */
#define SLAVE_ADDRES                         (0x1AU)
#define BUFFER_SIZE                          (0x05U)

/*******************************************************************************
Exported global variables and functions (to be accessed by other files)
*******************************************************************************/
unsigned short riic0_mode_flag;
unsigned short riic0_state;
unsigned char riic0_slave_address;
unsigned char* riic0_rx_address;
unsigned char* riic0_tx_address;
unsigned short riic0_rx_count;
unsigned short riic0_tx_count;
unsigned short riic0_rx_length;
unsigned char  riic0_rx_buffer[BUFFER_SIZE];
const unsigned char  riic0_tx_buffer[BUFFER_SIZE]="TEST";

void main(void);
void cgc_init(void);
void riic0_master_init(void);
void port_init(void);
void riic0_master_send(unsigned short adr , unsigned char * const tx_buf, unsigned short tx_num);
void riic0_master_receive(unsigned short adr , unsigned char * const rx_buf, unsigned short  rx_num);

/*******************************************************************************
* Function Name: main
* Description  : Main program
* Arguments    : none
* Return Value : none
*******************************************************************************/
void main(void)
{
    cgc_init();
    port_init();
    /* Start RIIC0 */
    riic0_master_init();

    /* Clear I2C interrupt flag */
    IR(RIIC0,RXI0) = 0U;
    IR(RIIC0,TXI0) = 0U;

    /* Enable I2C interrupt */
    IEN(RIIC0,TXI0) = 1U;
    IEN(RIIC0,TEI0) = 1U;
    IEN(RIIC0,RXI0) = 1U;
    IEN(RIIC0,EEI0) = 1U;

    /* Waiting for slave setup */
    while (1U == PORT2.PIDR.BIT.B1);
    riic0_master_send(SLAVE_ADDRES, riic0_tx_buffer, BUFFER_SIZE);

    while (1U == PORT2.PIDR.BIT.B1);
    riic0_master_receive(SLAVE_ADDRES, riic0_rx_buffer, BUFFER_SIZE);

    while(1U);
}

/*******************************************************************************
* Function Name: cgc_init
* Description  : Initialize CGC
* Arguments    : none
* Return Value : none
*******************************************************************************/
void cgc_init(void)
{
    unsigned long ii = 0U;

    /* Enable writing to registers related to operating modes, LPC, CGC and software reset */
    SYSTEM.PRCR.WORD = 0xA503;

    /* HOCO is not chosen. */
    /* Stop the HOCO. */
    SYSTEM.HOCOCR.BYTE = 0x01;
    /* Turn off power to HOCO. */
    SYSTEM.HOCOPCR.BYTE = 0x01;
    /* Main clock oscillator is chosen. Start it operating. */
    /* Wait 65,536 cycles * 12 MHz = 5.46 ms */
    SYSTEM.MOSCWTCR.BYTE = 0x0C;

    /* Set the main clock to operating. */
    SYSTEM.MOSCCR.BYTE = 0x00;

    while (0x00 != SYSTEM.MOSCCR.BYTE) 
    {
        /* Confirm that the written value can be read correctly. */
    }

    for(ii = 0; ii < 0x4FB; ii++)
    {
        /* Wait 12ms. See comment above for why. */
        nop() ;
    }

    /* Set the sub-clock to stopped. */
    SYSTEM.SOSCCR.BYTE = 0x01;

    /*
    Clock Description              Frequency
    ----------------------------------------
    Input Clock Frequency............. 12 MHz
    Peripheral Clock B Frequency...... 12 MHz
    System Clock Frequency............ 12 MHz */
    SYSTEM.SCKCR.LONG = 0x00000000;

    while (0x00000000 != SYSTEM.SCKCR.LONG) 
    {
         /* Confirm that the written value can be read correctly. */
    }

    /* Choose clock source (Main) */
    SYSTEM.SCKCR3.WORD = 0x0200;

    while (0x0200 != SYSTEM.SCKCR3.WORD) 
    {
        /* Confirm that the written value can be read correctly. */
    }

    /* LOCO is not chosen. This is the default out of reset. */
    SYSTEM.LOCOCR.BYTE = 0x01;

    /* Enable write protection */
    SYSTEM.PRCR.WORD = 0xA500;
}

/*******************************************************************************
* Function Name: port_init
* Description  : Initialize port
* Arguments    : none
* Return Value : none
*******************************************************************************/
void port_init(void)
{
    PORT2.PDR.BIT.B1 = 0U;  /* P21 Input */
    PORT2.PCR.BIT.B1 = 1U;  /* Input pull up enable */

    PORTD.PODR.BIT.B6 = 1U; /* PD6 High */
    PORTD.PDR.BIT.B6 = 1U;  /* PD6 Output */
}

/*******************************************************************************
* Function Name: riic0_master_init
* Description  : Initialize RIIC0 for master mode
* Arguments    : none
* Return Value : none
*******************************************************************************/
void riic0_master_init(void)
{
    /* Enable writing to registers related to operating modes, LPC, CGC and software reset */
    SYSTEM.PRCR.WORD = 0xA502;

    /* Enable writing to MPC pin function control registers */
    MPC.PWPR.BIT.B0WI  = 0U;
    MPC.PWPR.BIT.PFSWE = 1U;

    MSTP(RIIC0) = 0U;

    RIIC0.ICCR1.BIT.ICE    = 0U;
    RIIC0.ICCR1.BIT.IICRST = 1U;
    RIIC0.ICCR1.BIT.ICE    = 1U;

    /* Set ICSER */
    RIIC0.ICSER.BYTE = 0x09;

    /* Set transfer bit rate(100kbps) */
    RIIC0.ICMR1.BYTE = 0x10;
    RIIC0.ICBRL.BYTE = 0xFF;
    RIIC0.ICBRH.BYTE = 0xFF;

    /* Set ICMR2 and ICMR3 */
    RIIC0.ICMR2.BYTE = 0x00;
    RIIC0.ICMR3.BIT.NF = 0U;

    /* Set ICFER */
    RIIC0.ICFER.BYTE = 0x12;

    /* Set ICIER */
    RIIC0.ICIER.BYTE = 0xFE;

    /* Cancel internal reset */
    RIIC0.ICCR1.BIT.IICRST = 0U;

    /* Set interrupt priority */
    IPR(RIIC0, TXI0) = 0x0F;
    IPR(RIIC0, TEI0) = 0x0F;
    IPR(RIIC0, RXI0) = 0x0F;
    IPR(RIIC0, EEI0) = 0x0F;

    /* Set SCL0 pin */
    MPC.P12PFS.BYTE = 0x0FU;
    PORT1.PMR.BYTE |= 0x04U;

    /* Set SDA0 pin */
    MPC.P13PFS.BYTE = 0x0FU;
    PORT1.PMR.BYTE |= 0x08U;

    /* Disable writing to MPC pin function control registers */
    MPC.PWPR.BIT.PFSWE = 0U;
    MPC.PWPR.BIT.B0WI = 1U;

    /* Enable protection */
    SYSTEM.PRCR.WORD = 0xA500;
}

/*******************************************************************************
* Function Name: riic0_master_send
* Description  : Send data (Master sending mode)
* Arguments    : adr - slave address
*              : tx_buf - transmit buffer pointer
*              : tx_num - transmit data length
* Return Value : none
*******************************************************************************/
void riic0_master_send(unsigned short adr, unsigned char * const tx_buf, unsigned short tx_num)
{
    /* Bus Busy */
    if (1U == RIIC0.ICCR2.BIT.BBSY)
    {
        /* Do nothing */
    }
    else
    {
        /* Set parameter */
        riic0_tx_count = tx_num;
        riic0_tx_address = tx_buf;
        riic0_slave_address = adr & 0x7F;
        riic0_mode_flag = MASTER_TRANSMIT;
        riic0_state = MASTER_SENDS_ADR_7_W;

        RIIC0.ICCR2.BIT.ST = 1U;    /* Set start condition flag */
    }
}

/*******************************************************************************
* Function Name: riic0_master_receive
* Description  : Receive data (Master receiving mode)
* Arguments    : adr - slave address
*              : rx_buf - receive pointer
*              : rx_num - receive data length
* Return Value : none
*******************************************************************************/
void riic0_master_receive(unsigned short adr, unsigned char * const rx_buf, unsigned short rx_num)
{
    /* Bus Busy */
    if (1U == RIIC0.ICCR2.BIT.BBSY)
    {
        /* Do nothing */
    }
    else
    {
        /* Set parameter */
        riic0_rx_length = rx_num;
        riic0_rx_count = 0U;
        riic0_rx_address = rx_buf;
        riic0_slave_address = adr & 0x7F;
        riic0_mode_flag = MASTER_RECEIVE;
        riic0_state = MASTER_SENDS_ADR_7_R;

        RIIC0.ICCR2.BIT.ST = 1U;    /* Set start condition flag */
    }
}

/**********************************************************************************************************************
* Function Name: r_riic0_transmit_interrupt
* Description  : This function is TXI0 interrupt service routine.
* Arguments    : None
* Return Value : None
**********************************************************************************************************************/
#pragma interrupt r_riic0_transmit_interrupt(vect=VECT(RIIC0,TXI0))
static void r_riic0_transmit_interrupt(void)
{
    /* Master transmit */
    if (MASTER_TRANSMIT == riic0_mode_flag)
    {
        /* Master transmit address of slave(Master transmit mode) */
        if (MASTER_SENDS_ADR_7_W == riic0_state)
        {
            RIIC0.ICDRT = (riic0_slave_address << 1U) | MASTER_SENDS_ADR_7_W;
            riic0_state = MASTER_SENDS_DATA;
        }
        /* Master transmit data */
        else if (MASTER_SENDS_DATA == riic0_state)
        {
            /* continue to transmit */
            if (0U < riic0_tx_count)
            {
                RIIC0.ICDRT = *riic0_tx_address;
                riic0_tx_address++;
                riic0_tx_count--;
            }
            /* transmit end */
            else
            {
                riic0_state = MASTER_SENDS_END;
            }
        }
    }
    /* Master receive */
    else if (MASTER_RECEIVE == riic0_mode_flag)
    {
        /* Master transmit address of slave(Master receiving mode) */
        if (MASTER_SENDS_ADR_7_R == riic0_state)
        {
            RIIC0.ICDRT = (riic0_slave_address << 1U) | MASTER_SENDS_ADR_7_R;
            riic0_state = MASTER_RECEIVES_START;
        }
        /* Master receive data */
        else if (MASTER_RECEIVES_RESTART == riic0_state)
        {
            RIIC0.ICSR2.BIT.START = 0U;
            RIIC0.ICIER.BIT.STIE = 1U;
            RIIC0.ICCR2.BIT.RS = 1U;    /* Set restart condition flag */
        }
        else
        {
            /* Do nothing */
        }
    }
}

/**********************************************************************************************************************
* Function Name: r_riic0_transmitend_interrupt
* Description  : This function is TEI0 interrupt service routine.
* Arguments    : None
* Return Value : None
**********************************************************************************************************************/
#pragma interrupt r_riic0_transmitend_interrupt(vect=VECT(RIIC0,TEI0))
static void r_riic0_transmitend_interrupt(void)
{
    /* Master transmit complete */
    if (MASTER_SENDS_END == riic0_state)
    {
        RIIC0.ICSR2.BIT.STOP = 0U;
        RIIC0.ICCR2.BIT.SP = 1U;

        riic0_state = MASTER_SENDS_STOP;
    }
    else
    {
        /* Do nothing */
    }
}

/**********************************************************************************************************************
* Function Name: r_riic0_receive_interrupt
* Description  : This function is RXI0 interrupt service routine.
* Arguments    : None
* Return Value : None
**********************************************************************************************************************/
#pragma interrupt r_riic0_receive_interrupt(vect=VECT(RIIC0,RXI0))
static void r_riic0_receive_interrupt(void)
{
    volatile unsigned char dummy;

    /* Start master receiving */
    if (MASTER_RECEIVES_START == riic0_state)
    {
        /* receive 1 byte */
        if (1U == riic0_rx_length)
        {
            RIIC0.ICMR3.BIT.RDRFS = 1U;
            RIIC0.ICMR3.BIT.ACKWP = 1U;
            RIIC0.ICMR3.BIT.ACKBT = 1U;
            RIIC0.ICMR3.BIT.WAIT = 1U;
        }
        /* receive 2 byte */
        else if (2U == riic0_rx_length)
        {
            RIIC0.ICMR3.BIT.WAIT = 1U;
        }
        else
        {
            /* Do nothing */
        }

        /* Dummy read to release SCL */
        dummy = RIIC0.ICDRR;

        /* receive 1 byte */
        if (1U == riic0_rx_length)
        {
            riic0_state = MASTER_RECEIVES_STOPPING;
        }
        else
        {
            riic0_state = MASTER_RECEIVES_DATA;
        }
    }
    /* Master receiving */
    else if (MASTER_RECEIVES_DATA == riic0_state)
    {
        /* continue to receive */
        if (riic0_rx_count < riic0_rx_length)
        {
            /* receive last data -2 */
            if ((riic0_rx_count+3) == riic0_rx_length)
            {
                RIIC0.ICMR3.BIT.WAIT = 1U;

                *riic0_rx_address = RIIC0.ICDRR;
                riic0_rx_address++;
                riic0_rx_count++;
            }
            /* receive last data -1 */
            else if ((riic0_rx_count+2) == riic0_rx_length)
            {
                RIIC0.ICMR3.BIT.RDRFS = 1U;
                RIIC0.ICMR3.BIT.ACKWP = 1U;
                RIIC0.ICMR3.BIT.ACKBT = 1U;

                *riic0_rx_address = RIIC0.ICDRR;
                riic0_rx_address++;
                riic0_rx_count++;

                riic0_state = MASTER_RECEIVES_STOPPING;
            }
            /* continue to receive */
            else
            {
                *riic0_rx_address = RIIC0.ICDRR;
                riic0_rx_address++;
                riic0_rx_count++;
            }
        }
        else
        {
            /* Do nothing */
        }
    }
    /* Stop master receiving */
    else if (MASTER_RECEIVES_STOPPING == riic0_state)
    {
        RIIC0.ICSR2.BIT.STOP = 0U;
        RIIC0.ICCR2.BIT.SP = 1U;

        *riic0_rx_address = RIIC0.ICDRR;
        riic0_rx_address++;
        riic0_rx_count++;

        RIIC0.ICMR3.BIT.ACKWP = 1U;
        RIIC0.ICMR3.BIT.ACKBT = 1U;
        RIIC0.ICMR3.BIT.WAIT = 0U;

        /* send/receive OK ? */
        if ( 0 == strcmp((const char *)riic0_rx_buffer, riic0_tx_buffer) )
        {
            PORTD.PODR.BIT.B6 = 0U; /* Turn on LED1 */
        }
        else
        {
            /* Do nothing */
        }

        riic0_state = MASTER_RECEIVES_STOP;
    }
    else
    {
        /* Do nothing */
    }
}

/**********************************************************************************************************************
* Function Name: r_riic0_error_interrupt
* Description  : This function is EEI0 interrupt service routine.
* Arguments    : None
* Return Value : None
**********************************************************************************************************************/
#pragma interrupt r_riic0_error_interrupt(vect=VECT(RIIC0,EEI0))
static void r_riic0_error_interrupt(void)
{
    /* Master transmitg */
    if (MASTER_TRANSMIT == riic0_mode_flag)
    {
        /* Master transmit address of slave(Master transmit mode) */
        if (MASTER_SENDS_ADR_7_W == riic0_state)
        {
            RIIC0.ICSR2.BIT.START = 0U;
            RIIC0.ICIER.BIT.STIE = 0U;
            RIIC0.ICIER.BIT.SPIE = 1U;
        }
        /* Master transmit end */
        else if (MASTER_SENDS_STOP == riic0_state)
        {
            RIIC0.ICSR2.BIT.NACKF = 0U;
            RIIC0.ICSR2.BIT.STOP = 0U;
            RIIC0.ICIER.BIT.SPIE = 0U;
            RIIC0.ICIER.BIT.STIE = 1U;
        }
        else
        {
            /* Do nothing */
        }
    }
    /* Master receiving */
    else if (MASTER_RECEIVE == riic0_mode_flag)
    {
        /* Master transmit address of slave(Master receive mode) */
        if (MASTER_SENDS_ADR_7_R == riic0_state)
        {
            RIIC0.ICSR2.BIT.START = 0U;
            RIIC0.ICIER.BIT.STIE = 0U;
            RIIC0.ICIER.BIT.SPIE = 1U;
        }
        /* Master receive end */
        else if (MASTER_RECEIVES_STOP == riic0_state)
        {
            RIIC0.ICMR3.BIT.RDRFS = 0U;
            RIIC0.ICMR3.BIT.ACKWP = 1U;
            RIIC0.ICMR3.BIT.ACKBT = 0U;
            RIIC0.ICSR2.BIT.NACKF = 0U;
            RIIC0.ICSR2.BIT.STOP = 0U;
            RIIC0.ICIER.BIT.SPIE = 0U;
            RIIC0.ICIER.BIT.STIE = 1U;
        }
        else
        {
            /* Do nothing */
        }
    }
    else
    {
        /* Do nothing */
    }
}
